@@ -16,6 +16,22 @@ from requests_toolbelt.multipart.encoder import MultipartEncoder, MultipartEncod
 
 __version__ = pma.__version__
 
+pma_annotation_source_format_pathomation = 0
+pma_annotation_source_format_native = 1
+pma_annotation_source_format_visiopharm = 2
+pma_annotation_source_format_indicalabs = 3
+pma_annotation_source_format_aperio = 4
+pma_annotation_source_format_definiens = 4
+pma_annotation_source_format_xml = 4
+
+pma_annotation_target_format_pathomation = 2
+pma_annotation_target_format_visiopharm = 0
+pma_annotation_target_format_indicalabs = 1
+pma_annotation_target_format_aperio = 3
+pma_annotation_target_format_definiens = 3
+pma_annotation_target_format_xml = 3
+pma_annotation_target_format_csv = 4
+
 # internal module helper variables and functions
 _pma_sessions = dict()
 _pma_usernames = dict()
@@ -149,7 +165,7 @@ def get_version_info(pmacoreURL=_pma_pmacoreliteURL):
     # _pma_api_url() takes session information into account
 
     url = pma._pma_join(pmacoreURL, "api/json/GetVersionInfo")
-    if pma._pma_debug is True:
+    if pma._pma_debug == True:
         print(url)
 
     try:
@@ -171,7 +187,7 @@ def get_version_info(pmacoreURL=_pma_pmacoreliteURL):
 
 def get_api_version(pmacoreURL=_pma_pmacoreliteURL):
     url = pma._pma_join(pmacoreURL, "api/json/GetAPIVersion")
-    if pma._pma_debug is True:
+    if pma._pma_debug == True:
         print(url)
 
     try:
@@ -226,12 +242,11 @@ def connect(pmacoreURL=_pma_pmacoreliteURL, pmacoreUsername="", pmacorePassword=
     url = pma._pma_join(pmacoreURL, "api/json/authenticate?caller=SDK.Python")
     if (pmacoreUsername != ""):
         url += "&username=" + pma._pma_q(pmacoreUsername)
+        if (pma._pma_debug is True):
+            print("Authenticating via", url + "&password=TOP_SECRET")
     if (pmacorePassword != ""):
         url += "&password=" + pma._pma_q(pmacorePassword)
 
-    if (pma._pma_debug is True):
-        print(url)
-
     try:
         headers = {'Accept': 'application/json'}
         r = requests.get(url, headers=headers)
@@ -340,7 +355,7 @@ def get_directories(startDir, sessionID=None, recursive=False):
     """
     sessionID = _pma_session_id(sessionID)
     url = _pma_api_url(sessionID) + "GetDirectories?sessionID=" + pma._pma_q(sessionID) + "&path=" + pma._pma_q(startDir)
-    if pma._pma_debug is True:
+    if pma._pma_debug == True:
         print(url)
     r = requests.get(url)
     json = r.json()
@@ -374,7 +389,7 @@ def get_first_non_empty_directory(startDir=None, sessionID=None):
     try:
         slides = get_slides(startDir=startDir, sessionID=sessionID)
     except Exception:
-        if pma._pma_debug is True:
+        if pma._pma_debug == True:
             print("Unable to examine", startDir)
         if (startDir != "/"):
             return slides
@@ -391,7 +406,7 @@ def get_first_non_empty_directory(startDir=None, sessionID=None):
             try:
                 dirs = get_directories(startDir, sessionID)
             except Exception:
-                if pma._pma_debug is True:
+                if pma._pma_debug == True:
                     print("Unable to examine", startDir)
             else:
                 for dir in dirs:
@@ -409,6 +424,8 @@ def get_slides(startDir, sessionID=None, recursive=False):
     if (startDir.startswith("/")):
         startDir = startDir[1:]
     url = _pma_api_url(sessionID) + "GetFiles?sessionID=" + pma._pma_q(sessionID) + "&path=" + pma._pma_q(startDir)
+    if pma._pma_debug == True:
+        print(url)
     r = requests.get(url)
     json = r.json()
     global _pma_amount_of_data_downloaded
@@ -510,8 +527,8 @@ def get_uid(slideRef, sessionID=None):
     if ("Code" in json):
         raise Exception("get_uid on  " + slideRef + " resulted in: " + json["Message"])
     else:
-        fingerprint = json
-    return fingerprint
+        uid = json
+    return uid
 
 
 def get_fingerprint(slideRef, sessionID=None):
@@ -587,6 +604,8 @@ def get_slide_info(slideRef, sessionID=None):
 
     if (not (slideRef in _pma_slideinfos[sessionID])):
         url = _pma_api_url(sessionID) + "GetImageInfo?SessionID=" + pma._pma_q(sessionID) + "&pathOrUid=" + pma._pma_q(slideRef)
+        if pma._pma_debug == True:
+            print(url)
         r = requests.get(url)
         json = r.json()
         global _pma_amount_of_data_downloaded
@@ -597,6 +616,8 @@ def get_slide_info(slideRef, sessionID=None):
             _pma_slideinfos[sessionID][slideRef] = json["d"]
         else:
             _pma_slideinfos[sessionID][slideRef] = json
+    elif pma._pma_debug == True:
+        print("Getting slide info from cache")
 
     return _pma_slideinfos[sessionID][slideRef]
 
@@ -1003,7 +1024,27 @@ def get_annotations(slideRef, sessionID=None):
         annotations = ""
     return annotations
 
+def export_annotations(slideRef, annotation_source_format = [pma_annotation_source_format_pathomation], annotation_target_format = pma_annotation_target_format_xml, sessionID=None):
+    """
+    Retrieve the annotations for slide slideRef
+    """
+    sessionID = _pma_session_id(sessionID)
+    if (slideRef.startswith("/")):
+        slideRef = slideRef[1:]
+    source_format = "&source=" + pma._pma_q(",".join(annotation_source_format))
+    tgt_format = "&format=" + pma._pma_q(annotation_target_format)
+    url = _pma_api_url(sessionID) + "ExportAnnotations?sessionID=" + pma._pma_q(sessionID) + "&pathOrUid=" + pma._pma_q(slideRef) + tgt_format + source_format
+    if (pma._pma_debug is True):
+        print(url)
 
+    r = requests.get(url)
+
+    if (r.ok):
+        return BytesIO(r.content)
+    else:
+        raise ValueError("Unable to get annotations ("+r.status_code+")")
+    return None
+	
 def get_tiles(slideRef,
               fromX=0,
               fromY=0,
@@ -1061,7 +1102,7 @@ def show_slide(slideRef, sessionID=None):
             url += ("viewer/index.htm" + "?sessionID=" + pma._pma_q(sessionID) + "^&pathOrUid=" + pma._pma_q(slideRef)
                     )  # note the ^& to escape a regular &
 
-    if (pma._pma_debug is True):
+    if (pma._pma_debug == True):
         print(url)
     os.system(os_cmd + url)
 
@@ -1114,7 +1155,7 @@ def search_slides(startDir, pattern, sessionID=None):
 
     url = _pma_query_url(sessionID) + "Filename?sessionID=" + pma._pma_q(sessionID) + "&path=" + pma._pma_q(
         startDir) + "&pattern=" + pma._pma_q(pattern)
-    if pma._pma_debug is True:
+    if pma._pma_debug == True:
         print("url =", url)
 
     r = requests.get(url)
@@ -1221,10 +1262,10 @@ def upload(local_source_slide, target_folder, target_pma_core_sessionID, callbac
             raise Exception("Error uploading file {0}: {1}".format(f["Path"], r.json()["Message"]))
 
 
-def download(slide, save_directory=None, sessionID=None):
+def download(slideRef, save_directory=None, sessionID=None):
     """
         Downloads a slide from a PMA.core server.
-        :param str slide: The virtual path to the slide
+        :param str slideRef: The virtual path to the slide
         :param str save_directory: The local directory to save the downloaded files to
         :param str sessionID: The sessionID to authenticate to the pma.core server
     """
@@ -1239,25 +1280,27 @@ def download(slide, save_directory=None, sessionID=None):
             return None
         return fname[0].strip(";")
 
-    if not slide:
+    if not slideRef:
         raise ValueError("slide cannot be empty")
 
     if save_directory and not os.path.exists(save_directory):
         raise ValueError("The output directory does not exist {}".format(save_directory))
     
     sessionID = _pma_session_id(sessionID)
-    files = get_files_for_slide(slide, sessionID)
+    files = get_files_for_slide(slideRef, sessionID)
     if not files:
         raise ValueError("Slide not found")
     
-    mainDirectory = slide.rsplit('/', 1)[0]
+    mainDirectory = slideRef.rsplit('/', 1)[0]
    
     for f in files:
         relativePath = f.replace(mainDirectory, '').strip("\\").strip("/")
         pmaCoreDownloadUrl = _pma_url(sessionID) + "transfer/Download/"
 
-        print("Downloading file {} for slide {}".format(relativePath, slide))
-        params = {"sessionId": sessionID, "image": slide, "path": relativePath}
+        if pma._pma_debug == True:
+            print("Downloading file {} for slide {}".format(relativePath, slideRef))
+
+        params = {"sessionId": sessionID, "image": slideRef, "path": relativePath}
 
         with requests.get(pmaCoreDownloadUrl, params=params, stream=True) as r:
             r.raise_for_status()
@@ -1279,5 +1322,6 @@ def download(slide, save_directory=None, sessionID=None):
                         f.write(chunk)
                         progress = downloaded / total
                         if not prev or progress - prev > 0.05 or (progress - prev > 0 and downloaded == total):
-                            print("{0:.0%}".format(progress))
+                            if pma._pma_debug == True:
+                                print("{0:.0%}".format(progress))
                             prev = progress
